// $Id: combo.cc,v 1.16 1999/01/18 16:09:30 christof Exp $
/*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
 *  Copyright (C) 1998  Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "writers.hh"
#include "../Configuration.hh"

void Gtk_Combo::Include(File &f,const Widget &w)
{  f.include("gtk--/combo.h");
}

void Gtk_Combo::Declare(File &f,const Widget &w)
{  StandardWriter::Declare(f,w,"Gtk_Combo");
}

void Gtk_Combo::Construct(File &f,const Widget &w,bool is_member)
{  f << StandardWriter::ConstructName(f,w,is_member,"Gtk_Combo");
   f << "()";
}

bool Gtk_Combo::NeedConstruct(const Widget &w,bool is_member)
{  return false;
}

void Gtk_Combo::Customize(File &f,const Widget &w,const string instance)
{  Gtk_Widget::Customize(f,w,instance);
   // hmm perhaps call HBox Customize ??? since Combo is a HBox descendant
   // work around gtk bug (see gbwidget.c:3584)
   // "GTK BUG WORKAROUND - a combo should manage the size of its entry."
   int width=w.getInt("width",-1);
   int height=w.getInt("height",-1);
   if (width!=-1 || height!=-1)
   {  f << instance 
        << (Configuration.gtkmm_version<Gtkmm_Version(0,10,0) ? "entry." : "get_entry()->")
        << "set_usize(" 
        << (width>16?width-16:-1) << ", " << height << ");\n\t";
   }
   bool case_sensitive=w.getBool("case_sensitive",true);
   if (case_sensitive)
      f << instance << "set_case_sensitive(" << (case_sensitive?"true":"false")
        << ");\n\t";
   bool use_arrows=w.getBool("use_arrows",true);
   if (!use_arrows)
      f << instance << "set_use_arrows(" << (use_arrows?"true":"false")
        << ");\n\t";
   bool use_arrows_always=w.getBool("use_arrows_always",false);
   if (use_arrows_always)
      f << instance << "set_use_arrows_always(" << (use_arrows_always?"true":"false")
        << ");\n\t";
   // items
   if (w.hasTag("items")) // append them
   {  const string items(w.getString("items","<none>\n"));
      const string name(f.InstanceName(w.Name()));
      char *buffer=(char*)alloca(items.size()+1);
      memcpy(buffer,items.c_str(),items.size()+1);
      char *token,*stringp=buffer;
      string initial_choice;
      f << "{\tGList *gl=0;\n\t\t";
      if ((token=strsep(&stringp,"\n")))
      {  initial_choice=token;
         f << "gl=g_list_append(gl,(void*)\"" << f.CString(token) << "\");\n\t\t";
      }
      while (stringp && (token=strsep(&stringp,"\n")))
      {  if (!stringp && !token[0]) // do not use the last empty string
            break;
         f << "gl=g_list_append(gl,(void*)\"" << f.CString(token) << "\");\n\t\t";
      }
      f << instance << "set_popdown_strings(gl);\n"
           "\t\t""g_list_free(gl);\n"
           "\t}\n\t"
        << instance 
        << (Configuration.gtkmm_version<Gtkmm_Version(0,10,0) 
            ? "entry." : "get_entry()->")
        << "set_text(\"" << f.CString(initial_choice)
        << "\");\n\t";
   }
}

bool Gtk_Combo::SampleCode(File &f,const Widget &w,enum WriterType::CodePosition pos,const string misc)
{  if (! w.hasTag("items"))
   { switch(pos)
     { case WriterType::ContainingConstructor:
         {  f << "\t{\tGList *gl=0;\n"
              << "\t\tgl=g_list_append(gl,(void*)\"two\");\n"
              << "\t\tgl=g_list_append(gl,(void*)\"four\");\n"
              << "\t\tgl=g_list_append(gl,(void*)\"eight\");\n"
              << "\t\t" 
              << f.InstanceName(w.Name()) << (w.isPointer() ? "->" : "." ) 
              << "set_popdown_strings(gl);\n"
              << "\t\tg_list_free(gl);\n"
              << "\t\t"
              << f.InstanceName(w.Name()) << (w.isPointer() ? "->" : "." )
              << "entry.set_text(\"one\");\n"
              << "\t}\n";
   	 }
   	 break;
       default: break;
     }
   }
   return false;
}
