// $Id: searchcombo.cc,v 1.7 1999/01/18 15:38:32 christof Exp $
/*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
 *  Copyright (C) 1998  Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "writers.hh"
#include "../Configuration.hh"

void SearchComboBox::Include(File &f,const Widget &w)
{  f.include("SearchComboBox.h");
}

void SearchComboBox::Declare(File &f,const Widget &w)
{  StandardWriter::Declare(f,w,"SearchComboBox");
}

void SearchComboBox::Construct(File &f,const Widget &w,bool is_member)
{  f << StandardWriter::ConstructName(f,w,is_member,"SearchComboBox");
   bool alwaysfill=w.getBool("alwaysfill",true);
   f << "("<< (alwaysfill?"true":"false") <<")";
}

bool SearchComboBox::NeedConstruct(const Widget &w,bool is_member)
{  return true;
}

void SearchComboBox::Customize(File &f,const Widget &w,const string instance)
{  Gtk_Widget::Customize(f,w,instance);
   // hmm perhaps call HBox Customize ??? since Combo is a HBox descendant
   // work around gtk bug (see gbwidget.c:3584)
   // "GTK BUG WORKAROUND - a combo should manage the size of its entry."
   int width=w.getInt("width",-1);
   int height=w.getInt("height",-1);
   if (width!=-1 || height!=-1)
   {  f << instance 
        << (Configuration.gtkmm_version<Gtkmm_Version(0,10,0) 
            ? "entry." : "get_entry()->")
        << "set_usize(" << (width>16?width-16:-1) << ", " << height << ");\n\t";
   }
   bool case_sensitive=w.getBool("case_sensitive",true);
   if (case_sensitive)
      f << instance << "set_case_sensitive(" << (case_sensitive?"true":"false")
        << ");\n\t";
   bool use_arrows=w.getBool("use_arrows",true);
   if (!use_arrows)
      f << instance << "set_use_arrows(" << (use_arrows?"true":"false")
        << ");\n\t";
   bool use_arrows_always=w.getBool("use_arrows_always",false);
   if (use_arrows_always)
      f << instance << "set_use_arrows_always(" << (use_arrows_always?"true":"false")
        << ");\n\t";
}

bool SearchComboBox::SampleCode(File &f,const Widget &w,enum WriterType::CodePosition pos,const string misc)
{  // emit sample signal handler ???
   return false;
}

const string SearchComboBox::CallbackArgs(const Widget &w,const string signal,string &rettype)
{  if (signal=="search") 
   { rettype="SearchComboBox::searchresult";
     return "SearchComboBox *sc, const char *prefix, bool newsearch";
   }
   else if (signal=="selected") return "const char *text";
   else if (signal=="selected0") return "";
   return Gtk_Widget::CallbackArgs(w,signal,rettype); // eigentlich HBox
}
